@@ -1,14 +1,18 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
-
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Description: Mini shell using some of the LDAP functionalities of the library
+# Description:
+#   Mini shell using some of the LDAP functionalities of the library
 #
 # Author:
-#  Mathieu Gascon-Lefebvre (@mlefebvre)
+#   Mathieu Gascon-Lefebvre (@mlefebvre)
 #
 import re
 import string
@@ -132,11 +136,11 @@ class LdapShell(cmd.Cmd):
     def do_add_computer(self, line):
         args = shlex.split(line)
 
-        if not self.client.server.ssl:
+        if not self.client.server.ssl and not self.client.tls_started:
             print("Error adding a new computer with LDAP requires LDAPS.")
 
-        if len(args) != 1 and len(args) != 2:
-            raise Exception("Error expected a computer name and an optional password argument.")
+        if len(args) != 1 and len(args) != 2 and len(args) !=3:
+            raise Exception("Error expected a computer name, an optional password argument, and an optional nospns argument.")
 
         computer_name = args[0]
         if not computer_name.endswith('$'):
@@ -145,7 +149,7 @@ class LdapShell(cmd.Cmd):
         print("Attempting to add a new computer with the name: %s" % computer_name)
 
         password = ""
-        if len(args) == 1:
+        if len(args) == 1 or args[1] == "nospns":
             password = ''.join(random.choice(string.ascii_letters + string.digits + string.punctuation) for _ in range(15))
         else:
             password = args[1]
@@ -159,13 +163,35 @@ class LdapShell(cmd.Cmd):
         computer_hostname = computer_name[:-1] # Remove $ sign
         computer_dn = "CN=%s,CN=Computers,%s" % (computer_hostname, self.domain_dumper.root)
         print("New Computer DN: %s" % computer_dn)
-
-        spns = [
-            'HOST/%s' % computer_hostname,
-            'HOST/%s.%s' % (computer_hostname, domain),
-            'RestrictedKrbHost/%s' % computer_hostname,
-            'RestrictedKrbHost/%s.%s' % (computer_hostname, domain),
-        ]
+        
+        if len(args) == 3:
+            if args[2] == "nospns":
+                spns = [
+                    'HOST/%s.%s' % (computer_hostname, domain)
+                ]
+            else:
+                raise Exception("Invalid third argument: %s" %str(args[3]))     
+        elif len(args) == 2:
+            if args[1] != "nospns":
+                spns = [
+                    'HOST/%s' % computer_hostname,
+                    'HOST/%s.%s' % (computer_hostname, domain),
+                    'RestrictedKrbHost/%s' % computer_hostname,
+                    'RestrictedKrbHost/%s.%s' % (computer_hostname, domain),
+                ]
+            elif args[1] == "nospns":
+                spns = [
+                    'HOST/%s.%s' % (computer_hostname, domain)
+                ]
+        elif len(args) == 1:
+            spns = [
+                'HOST/%s' % computer_hostname,
+                'HOST/%s.%s' % (computer_hostname, domain),
+                'RestrictedKrbHost/%s' % computer_hostname,
+                'RestrictedKrbHost/%s.%s' % (computer_hostname, domain),
+            ]
+        else:
+            raise Exception("Invalid third argument: %s" %str(self.args[3])) 
         ucd = {
             'dnsHostName': '%s.%s' % (computer_hostname, domain),
             'userAccountControl': 4096,
@@ -184,8 +210,45 @@ class LdapShell(cmd.Cmd):
         else:
             print('Adding new computer with username: %s and password: %s result: OK' % (computer_name, password))
 
+    def do_rename_computer(self, line):
+        args = shlex.split(line)
+
+        if len(args) != 2:
+            raise Exception("Current Computer sAMAccountName and New Computer sAMAccountName required (rename_computer comp1$ comp2$).")
+
+        current_name = args[0]
+
+        new_name = args[1]
+
+        self.client.search(self.domain_dumper.root, '(sAMAccountName=%s)' % escape_filter_chars(current_name), attributes=['objectSid', 'sAMAccountName'])
+        computer_dn = self.client.entries[0].entry_dn
+        
+        if not computer_dn:
+            raise Exception("Computer not found in LDAP: %s" % current_name)
+
+        entry = self.client.entries[0]
+        samAccountName = entry["samAccountName"].value
+        print("Original sAMAccountName: %s" % samAccountName)
+
+        print("New sAMAccountName: %s" % new_name)
+        self.client.modify(computer_dn, {'sAMAccountName':(ldap3.MODIFY_REPLACE, [new_name])})
+        
+        if self.client.result["result"] == 0:
+            print("Updated sAMAccountName successfully")
+        else:
+            if self.client.result['result'] == 50:
+                raise Exception('Could not modify object, the server reports insufficient rights: %s', self.client.result['message'])
+            elif self.client.result['result'] == 19:
+                raise Exception('Could not modify object, the server reports a constrained violation: %s', self.client.result['message'])
+            else:
+                raise Exception('The server returned an error: %s', self.client.result['message'])
+
     def do_add_user(self, line):
         args = shlex.split(line)
+
+        if not self.client.server.ssl and not self.client.tls_started:
+            print("Error adding a new user with LDAP requires LDAPS.")
+
         if len(args) == 0:
             raise Exception("A username is required.")
 
@@ -300,6 +363,16 @@ class LdapShell(cmd.Cmd):
         self.domain_dumper.domainDump()
         print('Domain info dumped into lootdir!')
 
+    def do_start_tls(self, line):
+        if not self.client.tls_started and not self.client.server.ssl:
+            print('Sending StartTLS command...')
+            if not self.client.start_tls():
+                raise Exception("StartTLS failed")
+            else:
+                print('StartTLS succeded, you are now using LDAPS!')
+        else:
+            print('It seems you are already connected through a TLS channel.')
+
     def do_disable_account(self, username):
         self.toggle_account_enable_disable(username, False)
 
@@ -433,51 +506,53 @@ class LdapShell(cmd.Cmd):
 
     def do_grant_control(self, line):
         args = shlex.split(line)
+        if len(args) == 2:
+            target_spec, grantee_name = args
+            target_base = self.domain_dumper.root
+        elif len(args) == 3:
+            target_base, target_spec, grantee_name = args
+        else:
+            raise Exception(f'Expecting target and grantee or search base, target and grantee. Received {len(args)} arguments instead.')
 
-        if len(args) != 1 and len(args) != 2:
-            raise Exception("Error expecting target and grantee names for RBCD attack. Recieved %d arguments instead." % len(args))
+        if target_spec.startswith('(') and target_spec.endswith(')'):
+            target_filter = target_spec
+        else:
+            target_filter = f'(sAMAccountName={escape_filter_chars(target_spec)})'
 
         controls = security_descriptor_control(sdflags=0x04)
 
-        target_name = args[0]
-        grantee_name = args[1]
+        self.client.search(self.domain_dumper.root, f'(sAMAccountName={escape_filter_chars(grantee_name)})', attributes=['objectSid'], controls=controls)
+        if not self.client.entries:
+            raise Exception('Grantee not found')
+        if len(self.client.entries) > 1:
+            raise Exception('Grantee not unique')
+        grantee_sid = self.client.entries[0]['objectSid'].value
+        print(f'Resolved {grantee_name!r} to {grantee_sid!r}')
 
-        success = self.client.search(self.domain_dumper.root, '(sAMAccountName=%s)' % escape_filter_chars(target_name), attributes=['objectSid', 'nTSecurityDescriptor'], controls=controls)
-        if success is False or len(self.client.entries) != 1:
-            raise Exception("Error expected only one search result got %d results", len(self.client.entries))
-
-        target = self.client.entries[0]
-        target_sid = target["objectSid"].value
-        print("Found Target DN: %s" % target.entry_dn)
-        print("Target SID: %s\n" % target_sid)
-
-        success = self.client.search(self.domain_dumper.root, '(sAMAccountName=%s)' % escape_filter_chars(grantee_name), attributes=['objectSid'])
-        if success is False or len(self.client.entries) != 1:
-            raise Exception("Error expected only one search result got %d results", len(self.client.entries))
-
-        grantee = self.client.entries[0]
-        grantee_sid = grantee["objectSid"].value
-        print("Found Grantee DN: %s" % grantee.entry_dn)
-        print("Grantee SID: %s" % grantee_sid)
+        self.client.search(target_base, target_filter, attributes=['nTSecurityDescriptor'], controls=controls)
+        if not self.client.entries:
+            raise Exception('Target not found')
+        if len(self.client.entries) > 1:
+            raise Exception('Target not unique')
+        target_entry = self.client.entries[0]
+        print(f'Resolved {target_filter!r} to {target_entry.entry_dn!r}')
 
         try:
-            sd = ldaptypes.SR_SECURITY_DESCRIPTOR(data=target['nTSecurityDescriptor'].raw_values[0])
+            sd = ldaptypes.SR_SECURITY_DESCRIPTOR(data=target_entry['nTSecurityDescriptor'].raw_values[0])
         except IndexError:
             sd = self.create_empty_sd()
-
         sd['Dacl'].aces.append(self.create_allow_ace(grantee_sid))
-        self.client.modify(target.entry_dn, {'nTSecurityDescriptor':[ldap3.MODIFY_REPLACE, [sd.getData()]]}, controls=controls)
 
+        self.client.modify(target_entry.entry_dn, {'nTSecurityDescriptor': [ldap3.MODIFY_REPLACE, [sd.getData()]]}, controls=controls)
         if self.client.result['result'] == 0:
             print('DACL modified successfully!')
-            print('%s now has control of %s' % (grantee_name, target_name))
+            print(f'{grantee_name!r} now has control of {target_entry.entry_dn!r}')
+        elif self.client.result['result'] == 50:
+            raise Exception(f'Could not modify object, the server reports insufficient rights: {self.client.result["message"]}')
+        elif self.client.result['result'] == 19:
+            raise Exception(f'Could not modify object, the server reports a constrained violation: {self.client.result["message"]}')
         else:
-            if self.client.result['result'] == 50:
-                raise Exception('Could not modify object, the server reports insufficient rights: %s', self.client.result['message'])
-            elif self.client.result['result'] == 19:
-                raise Exception('Could not modify object, the server reports a constrained violation: %s', self.client.result['message'])
-            else:
-                raise Exception('The server returned an error: %s', self.client.result['message'])
+            raise Exception(f'The server returned an error: {self.client.result["message"]}')
 
     def do_set_rbcd(self, line):
         args = shlex.split(line)
@@ -557,6 +632,30 @@ class LdapShell(cmd.Cmd):
         except IndexError:
             return None
 
+    def do_whoami(self, line):
+        print(self.client.extend.standard.who_am_i())
+
+    def do_dirsync(self, line):
+        arguments = shlex.split(line)
+        if len(arguments) == 0:
+            raise Exception("A query is required.")
+
+        domain_dn = self.domain_dumper.root
+        sync_filter = arguments[0]
+        attributes = list(set(['name', 'sAMAccountName', 'objectsid'] + arguments[1:]))
+        
+        sync = self.client.extend.microsoft.dir_sync(domain_dn, attributes=attributes, sync_filter=sync_filter, incremental_values=False)
+
+        results = []
+        while sync.more_results:
+            results += sync.loop()
+
+        for result in results:
+            print(result['dn'])
+            for k, v in result['attributes'].items():
+                print(k, v)
+            print()
+
     def do_exit(self, line):
         if self.shell is not None:
             self.shell.close()
@@ -564,7 +663,8 @@ class LdapShell(cmd.Cmd):
 
     def do_help(self, line):
         print("""
- add_computer computer [password] - Adds a new computer to the domain with the specified password. Requires LDAPS.
+ add_computer computer [password] [nospns] - Adds a new computer to the domain with the specified password. If nospns is specified, computer will be created with only a single necessary HOST SPN. Requires LDAPS.
+ rename_computer current_name new_name - Sets the SAMAccountName attribute on a computer object to a new value.
  add_user new_user [parent] - Creates a new user.
  add_user_to_group user group - Adds a user to a group.
  change_password user [password] - Attempt to change a given user's password. Requires LDAPS.
@@ -576,10 +676,13 @@ class LdapShell(cmd.Cmd):
  get_user_groups user - Retrieves all groups this user is a member of.
  get_group_users group - Retrieves all members of a group.
  get_laps_password computer - Retrieves the LAPS passwords associated with a given computer (sAMAccountName).
- grant_control target grantee - Grant full control of a given target object (sAMAccountName) to the grantee (sAMAccountName).
+ grant_control [search_base] target grantee - Grant full control on a given target object (sAMAccountName or search filter, optional search base) to the grantee (sAMAccountName).
  set_dontreqpreauth user true/false - Set the don't require pre-authentication flag to true or false.
  set_rbcd target grantee - Grant the grantee (sAMAccountName) the ability to perform RBCD to the target (sAMAccountName).
+ start_tls - Send a StartTLS command to upgrade from LDAP to LDAPS. Use this to bypass channel binding for operations necessitating an encrypted channel.
  write_gpo_dacl user gpoSID - Write a full control ACE to the gpo for the given user. The gpoSID must be entered surrounding by {}.
+ whoami - get connected user
+ dirsync - Dirsync requested attributes
  exit - Terminates this session.""")
 
     def do_EOF(self, line):
